@@ -44,7 +44,8 @@ ctcp_reply (session *sess, char *nick, char *word[], char *word_eol[],
 	conf = strdup (conf);
 	/* process %C %B etc */
 	check_special_chars (conf, TRUE);
-	auto_insert (tbuf, sizeof (tbuf), conf, word, word_eol, "", "", word_eol[5], "", "", nick);
+	auto_insert (tbuf, sizeof (tbuf), conf, word, word_eol, "", "", word_eol[5],
+					 server_get_network (sess->server, TRUE), "", "", nick);
 	free (conf);
 	handle_command (sess, tbuf, FALSE);
 }
