@@ -167,9 +167,15 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text, int id)
 				sess = serv->server_session;
 		}
 
-		if (prefs.beepmsg || (sess && sess->beep))
+		if (prefs.input_beep_priv || (sess && sess->beep))
 			sound_beep (sess);
 
+		if (sess && sess->tray)
+			fe_tray_set_icon (FE_ICON_MESSAGE);
+
+		if (prefs.input_flash_priv)
+			fe_flash_window (sess);
+
 		if (ip && ip[0])
 		{
 			if (prefs.logging && sess->logfd != -1 &&
@@ -192,16 +198,19 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text, int id)
 	{
 		sess = serv->front_session;
 
-		if (prefs.beepmsg || (sess && sess->beep))
+		if (prefs.input_beep_priv || (sess && sess->beep))
 			sound_beep (sess);
 
 		EMIT_SIGNAL (XP_TE_PRIVMSG, sess, from, text, idtext, NULL, 0);
 		return;
 	}
 
-	if (prefs.beepmsg || sess->beep)
+	if (prefs.input_beep_priv || sess->beep)
 		sound_beep (sess);
 
+	if (prefs.input_flash_priv)
+		fe_flash_window (sess);
+
 	if (sess->type == SESS_DIALOG)
 		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, idtext, NULL, 0);
 	else
@@ -304,11 +313,11 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme, i
 		if (is_channel (serv, chan))
 		{
 			sess = find_channel (serv, chan);
-			beep = prefs.beepchans;
+			beep = prefs.input_beep_chans;
 		} else
 		{
 			/* it's a private action! */
-			beep = prefs.beepmsg;
+			beep = prefs.input_beep_priv;
 			/* find a dialog tab for it */
 			sess = find_dialog (serv, from);
 			/* if non found, open a new one */
@@ -343,12 +352,19 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme, i
 	if (!fromme)
 	{
 		hilight = is_hilight (from, text, sess, serv);
-		if (hilight && prefs.beephilight)
+		if (hilight && prefs.input_beep_hilight)
 			beep = TRUE;
 
 		if (beep || sess->beep)
 			sound_beep (sess);
 
+		if (sess->tray)
+			fe_tray_set_icon (FE_ICON_MESSAGE);
+
+		/* private action, flash? */
+		if (!is_channel (serv, chan) && prefs.input_flash_priv)
+			fe_flash_window (sess);
+
 		if (hilight)
 		{
 			EMIT_SIGNAL (XP_TE_HCHANACTION, sess, from, text, nickchar, NULL, 0);
@@ -356,7 +372,10 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme, i
 		}
 	}
 
-	EMIT_SIGNAL (XP_TE_CHANACTION, sess, from, text, nickchar, NULL, 0);
+	if (fromme)
+		EMIT_SIGNAL (XP_TE_UACTION, sess, from, text, nickchar, NULL, 0);
+	else
+		EMIT_SIGNAL (XP_TE_CHANACTION, sess, from, text, nickchar, NULL, 0);
 }
 
 void
@@ -406,15 +425,26 @@ inbound_chanmsg (server *serv, session *sess, char *chan, char *from, char *text
 	inbound_make_idtext (serv, idtext, sizeof (idtext), id);
 
 	if (sess->type != SESS_DIALOG)
-		if (prefs.beepchans || sess->beep)
+	{
+		if (prefs.input_beep_chans || sess->beep)
 			sound_beep (sess);
 
+		if (sess->tray)
+			fe_tray_set_icon (FE_ICON_MESSAGE);
+	}
+
 	if (is_hilight (from, text, sess, serv))
 	{
 		hilight = TRUE;
-		if (prefs.beephilight)
+		if (prefs.input_beep_hilight)
 			sound_beep (sess);
 	}
+	else
+	{
+		if (sess->type != SESS_DIALOG && prefs.input_flash_chans)
+			fe_flash_window (sess);
+	}
+
 	if (sess->type == SESS_DIALOG)
 		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, idtext, NULL, 0);
 	else if (hilight)
@@ -771,8 +801,8 @@ inbound_ping_reply (session *sess, char *timestring, char *from)
 	if (lag)
 	{
 		sess->server->lag_sent = 0;
-		snprintf (outbuf, sizeof (outbuf), "%ld.%ld", dif / 100000, (dif / 10000) % 100);
-		fe_set_lag (sess->server, (int)((float)atof (outbuf)));
+		sess->server->lag = dif / 1000;
+		fe_set_lag (sess->server, dif / 100000);
 		return;
 	}
 
