@@ -415,6 +415,11 @@ plugin_auto_load_cb (char *filename)
 {
 	char *pMsg;
 
+#ifndef WIN32	/* black listed */
+	if (!strcmp (file_part (filename), "dbus.so"))
+		return;
+#endif
+
 	pMsg = plugin_load (ps, filename, NULL);
 	if (pMsg)
 	{
@@ -610,7 +615,7 @@ plugin_timeout_cb (xchat_hook *hook)
 	ret = ((xchat_timer_cb *)hook->callback) (hook->userdata);
 
 	/* the callback might have already unhooked it! */
-	if (!g_slist_find (hook_list, hook))
+	if (!g_slist_find (hook_list, hook) || hook->type == HOOK_DELETED)
 		return 0;
 
 	if (ret == 0)
@@ -754,7 +759,7 @@ void *
 xchat_unhook (xchat_plugin *ph, xchat_hook *hook)
 {
 	/* perl.c trips this */
-	if (hook->type == HOOK_DELETED || !g_slist_find (hook_list, hook))
+	if (!g_slist_find (hook_list, hook) || hook->type == HOOK_DELETED)
 		return NULL;
 
 	if (hook->type == HOOK_TIMER && hook->tag != 0)
@@ -961,8 +966,17 @@ const char *
 xchat_get_info (xchat_plugin *ph, const char *id)
 {
 	session *sess;
-	guint32 hash = str_hash (id);
+	guint32 hash;
 
+	/*                 1234567890 */
+	if (!strncmp (id, "event_text", 10))
+	{
+		char *e = (char *)id + 10;
+		if (*e == ' ') e++;	/* 2.8.0 only worked without a space */
+		return text_find_format_string (e);
+	}
+
+	hash = str_hash (id);
 	/* do the session independant ones first */
 	switch (hash)
 	{
@@ -1014,6 +1028,9 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 	case 0x1c0e99c1: /* inputbox */
 		return fe_get_inputbox_contents (sess);
 
+	case 0x633fb30:	/* modes */
+		return sess->current_modes;
+
 	case 0x6de15a2e:	/* network */
 		return server_get_network (sess->server, FALSE);
 
@@ -1394,7 +1411,9 @@ xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 		case 0xd1b:	/* id */
 			return ((struct session *)data)->server->id;
 		case 0x5cfee87:	/* flags */
-			tmp = ((struct session *)data)->beep;            /* bit 8 */
+			tmp = ((struct session *)data)->tray;            /* bit 9 */
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->beep;               /* 8 */
 			tmp <<= 1;
 			tmp |= ((struct session *)data)->color_paste;        /* 7 */
 			tmp <<= 1;
