@@ -61,10 +61,10 @@ irc_nickserv (server *serv, char *cmd, char *arg1, char *arg2, char *arg3)
 		tcp_sendf (serv, "PRIVMSG NICKSERV :%s %s%s%s\r\n", cmd, arg1, arg2, arg3);
 		break;
 	case 1:
-		tcp_sendf (serv, "NICKSERV :%s %s%s%s\r\n", cmd, arg1, arg2, arg3);
+		tcp_sendf (serv, "NICKSERV %s %s%s%s\r\n", cmd, arg1, arg2, arg3);
 		break;
 	case 2:
-		tcp_sendf (serv, "NS :%s %s%s%s\r\n", cmd, arg1, arg2, arg3);
+		tcp_sendf (serv, "NS %s %s%s%s\r\n", cmd, arg1, arg2, arg3);
 		break;
 	case 3:
 		tcp_sendf (serv, "PRIVMSG NS :%s %s%s%s\r\n", cmd, arg1, arg2, arg3);
@@ -241,7 +241,7 @@ irc_topic (server *serv, char *channel, char *topic)
 }
 
 static void
-irc_list_channels (server *serv, char *arg)
+irc_list_channels (server *serv, char *arg, int min_users)
 {
 	if (arg[0])
 	{
@@ -250,8 +250,7 @@ irc_list_channels (server *serv, char *arg)
 	}
 
 	if (serv->use_listargs)
-								/* 1234567890123456 */
-		tcp_send_len (serv, "LIST >0,<10000\r\n", 16);
+		tcp_sendf (serv, "LIST >%d,<10000\r\n", min_users - 1);
 	else
 		tcp_send_len (serv, "LIST\r\n", 6);
 }
@@ -406,6 +405,7 @@ process_numeric (session * sess, int n,
 			}
 
 			serv->skip_next_userhost = FALSE;
+			break;
 		}
 		else goto def;
 
