@@ -225,10 +225,17 @@ log_create_pathname (char *servname, char *channame, char *netname)
 	strftime (fnametime, sizeof (fnametime), fname, tm);
 
 	/* create final path/filename */
-	snprintf (fname, sizeof (fname), "%s/xchatlogs/%s", get_xdir_utf8 (), fnametime);
+#ifdef WIN32
+	if (fnametime[0] == '/' || (fnametime[0] >= 'A' && fnametime[1] == ':'))
+#else
+	if (fnametime[0] == '/')	/* is it fullpath already? */
+#endif
+		snprintf (fname, sizeof (fname), "%s", fnametime);
+	else
+		snprintf (fname, sizeof (fname), "%s/xchatlogs/%s", get_xdir_utf8 (), fnametime);
 
 	/* now we need it in FileSystem encoding */
-	fs = g_filename_from_utf8 (fname, -1, 0, 0, 0);
+	fs = xchat_filename_from_utf8 (fname, -1, 0, 0, 0);
 
 	/* create all the subdirectories */
 	if (fs)
@@ -641,6 +648,7 @@ static char * const pevt_chanmsg_help[] = {
 static char * const pevt_privmsg_help[] = {
 	N_("Nickname"),
 	N_("The message"),
+	N_("Identified text")
 };
 
 static char * const pevt_changenick_help[] = {
@@ -705,12 +713,6 @@ static char * const pevt_channotice_help[] = {
 	N_("The message"),
 };
 
-static char * const pevt_dprivmsg_help[] = {
-	N_("Nickname"),
-	N_("The message"),
-	N_("Identified text"),
-};
-
 static char * const pevt_uchangenick_help[] = {
 	N_("Old nickname"),
 	N_("New nickname"),
@@ -923,12 +925,13 @@ static char * const pevt_connect_help[] = {
 };
 
 static char * const pevt_sconnect_help[] = {
-	N_("PID"),
+	"PID"
 };
 
 static char * const pevt_generic_nick_help[] = {
 	N_("Nickname"),
-	N_("Server Name")
+	N_("Server Name"),
+	N_("Network")
 };
 
 static char * const pevt_chanmodes_help[] = {
@@ -1113,8 +1116,8 @@ pevent_load_defaults ()
 		if (pntevts_text[i])
 			free (pntevts_text[i]);
 
-		/* don't gettext() the blank ones */
-		if (i == XP_TE_OPENDIALOG || i == XP_TE_BEEP)
+		/* make-te.c sets this 128 flag (DON'T call gettext() flag) */
+		if (te[i].num_args & 128)
 			pntevts_text[i] = strdup (te[i].def);
 		else
 			pntevts_text[i] = strdup (_(te[i].def));
@@ -1124,7 +1127,7 @@ pevent_load_defaults ()
 void
 pevent_make_pntevts ()
 {
-	int i, m, len;
+	int i, m;
 	char out[1024];
 
 	for (i = 0; i < NUM_XP; i++)
@@ -1137,9 +1140,11 @@ pevent_make_pntevts ()
 						 _("Error parsing event %s.\nLoading default."), te[i].name);
 			fe_message (out, FE_MSG_WARN);
 			free (pntevts_text[i]);
-			len = strlen (te[i].def) + 1;
-			pntevts_text[i] = malloc (len);
-			memcpy (pntevts_text[i], te[i].def, len);
+			/* make-te.c sets this 128 flag (DON'T call gettext() flag) */
+			if (te[i].num_args & 128)
+				pntevts_text[i] = strdup (te[i].def);
+			else
+				pntevts_text[i] = strdup (_(te[i].def));
 			if (pevt_build_string (pntevts_text[i], &(pntevts[i]), &m) != 0)
 			{
 				fprintf (stderr,
@@ -1306,7 +1311,7 @@ pevent_load (char *filename)
 static void
 pevent_check_all_loaded ()
 {
-	int i, len;
+	int i;
 
 	for (i = 0; i < NUM_XP; i++)
 	{
@@ -1315,9 +1320,11 @@ pevent_check_all_loaded ()
 			/*printf ("%s\n", te[i].name);
 			snprintf(out, sizeof(out), "The data for event %s failed to load. Reverting to defaults.\nThis may be because a new version of XChat is loading an old config file.\n\nCheck all print event texts are correct", evtnames[i]);
 			   gtkutil_simpledialog(out); */
-			len = strlen (te[i].def) + 1;
-			pntevts_text[i] = malloc (len);
-			memcpy (pntevts_text[i], te[i].def, len);
+			/* make-te.c sets this 128 flag (DON'T call gettext() flag) */
+			if (te[i].num_args & 128)
+				pntevts_text[i] = strdup (te[i].def);
+			else
+				pntevts_text[i] = strdup (_(te[i].def));
 		}
 	}
 }
@@ -1348,7 +1355,7 @@ format_event (session *sess, int index, char **args, char *o, int sizeofo, unsig
 	char *i, *ar, d, a, done_all = FALSE;
 
 	i = pntevts[index];
-	numargs = te[index].num_args;
+	numargs = te[index].num_args & 0x7f;
 
 	oi = ii = len = d = a = 0;
 	o[0] = 0;
@@ -1660,6 +1667,18 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 		display_event (sess, index, word, stripcolor_args);
 }
 
+char *
+text_find_format_string (char *name)
+{
+	int i = 0;
+
+	i = pevent_find (name, &i);
+	if (i >= 0)
+		return pntevts_text[i];
+
+	return NULL;
+}
+
 int
 text_emit_by_name (char *name, session *sess, char *a, char *b, char *c, char *d)
 {
@@ -1778,7 +1797,7 @@ sound_play (const char *file, gboolean quiet)
 	}
 	wavfile[sizeof (wavfile) - 1] = 0;	/* ensure termination */
 
-	file_fs = g_filename_from_utf8 (wavfile, -1, 0, 0, 0);
+	file_fs = xchat_filename_from_utf8 (wavfile, -1, 0, 0, 0);
 	if (!file_fs)
 		return;
 
