@@ -273,7 +273,7 @@ path_part (char *file, char *path, int pathlen)
 }
 
 char *				/* like strstr(), but nocase */
-nocasestrstr (char *s, char *wanted)
+nocasestrstr (const char *s, const char *wanted)
 {
 	register const int len = strlen (wanted);
 
@@ -1079,7 +1079,6 @@ static const domain_t domain[] =
 		{"UA", N_("Ukraine") },
 		{"UG", N_("Uganda") },
 		{"UK", N_("United Kingdom") },
-		{"UM", N_("US Minor Outlying Islands") },
 		{"US", N_("United States of America") },
 		{"UY", N_("Uruguay") },
 		{"UZ", N_("Uzbekistan") },
@@ -1178,7 +1177,8 @@ int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPtr) {
 		if (*src != quote) *buf++ = '\\';
 	    }
 	    *buf++ = *src;
-	} else if (isspace((unsigned char) *src)) {
+	/*} else if (isspace((unsigned char) *src)) {*/
+	} else if (*src == ' ') {
 	    if (*argv[argc]) {
 		buf++, argc++;
 		if (argc == argvAlloced) {
@@ -1233,7 +1233,7 @@ int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPtr) {
 }
 
 int
-util_exec (char *cmd)
+util_exec (const char *cmd)
 {
 	int pid;
 	char **argv;
@@ -1265,6 +1265,31 @@ util_exec (char *cmd)
 #endif
 }
 
+int
+util_execv (char * const argv[])
+{
+	int pid, fd;
+
+#ifndef WIN32
+	pid = fork ();
+	if (pid == -1)
+		return -1;
+	if (pid == 0)
+	{
+		/* Now close all open file descriptors except stdin, stdout and stderr */
+		for (fd = 3; fd < 1024; fd++) close(fd);
+		execv (argv[0], argv);
+		_exit (0);
+	} else
+	{
+		return pid;
+	}
+#else
+	spawnv (_P_DETACH, argv[0], argv);
+	return 0;
+#endif
+}
+
 unsigned long
 make_ping_time (void)
 {
