@@ -76,6 +76,12 @@ int xchat_is_quitting = FALSE;
 int arg_dont_autoconnect = FALSE;
 int arg_skip_plugins = FALSE;
 char *arg_url = NULL;
+gint arg_existing = FALSE;
+
+#ifdef USE_DBUS
+#include "dbus/dbus-client.h"
+#include "dbus/dbus-plugin.h"
+#endif /* USE_DBUS */
 
 struct session *current_tab;
 struct session *current_sess = 0;
@@ -295,6 +301,11 @@ irc_init (session *sess)
 	if (!arg_skip_plugins)
 		plugin_auto_load (sess);	/* autoload ~/.xchat *.so */
 #endif
+
+#ifdef USE_DBUS
+	plugin_add (sess, NULL, NULL, dbus_plugin_init, NULL, NULL, FALSE);
+#endif
+
 	plugin_add (sess, NULL, NULL, timer_plugin_init, NULL, NULL, FALSE);
 
 	if (prefs.notify_timeout)
@@ -433,7 +444,8 @@ send_quit_or_part (session * killsess)
 			}
 		} else
 		{
-			if (killsess->type == SESS_CHANNEL && killsess->channel[0])
+			if (killsess->type == SESS_CHANNEL && killsess->channel[0] &&
+				 !killserv->sent_quit)
 			{
 				server_sendpart (killserv, killsess->channel, 0);
 			}
@@ -497,8 +509,12 @@ session_free (session *killsess)
 
 	fe_session_callback (killsess);
 
-	if (current_sess == killsess && sess_list)
-		current_sess = sess_list->data;
+	if (current_sess == killsess)
+	{
+		current_sess = NULL;
+		if (sess_list)
+			current_sess = sess_list->data;
+	}
 
 	free (killsess);
 
@@ -547,12 +563,12 @@ static char defaultconf_commands[] =
 	"NAME AME\n"			"CMD allchan me &2\n\n"\
 	"NAME ANICK\n"			"CMD allserv nick &2\n\n"\
 	"NAME AMSG\n"			"CMD allchan say &2\n\n"\
-	"NAME BACK\n"			"CMD away\n\n"\
 	"NAME BANLIST\n"		"CMD quote MODE %c +b\n\n"\
 	"NAME CHAT\n"			"CMD dcc chat %2\n\n"\
 	"NAME DIALOG\n"		"CMD query %2\n\n"\
 	"NAME DMSG\n"			"CMD msg =%2 &3\n\n"\
 	"NAME EXIT\n"			"CMD quit\n\n"\
+	"NAME GREP\n"			"CMD lastlog -r &2\n\n"\
 	"NAME J\n"				"CMD join &2\n\n"\
 	"NAME KILL\n"			"CMD quote KILL %2 :&3\n\n"\
 	"NAME LEAVE\n"			"CMD part &2\n\n"\
@@ -899,7 +915,7 @@ xchat_init (void)
 		"NAME %s\n"		"CMD op %%a\n\n"
 		"NAME %s\n"		"CMD deop %%a\n\n"
 		"NAME %s\n"		"CMD ban %%s\n\n"
-		"NAME %s\n"		"CMD getstr %s \"kick %%s\" \"%s\"\n\n"
+		"NAME %s\n"		"CMD getstr \"%s\" \"kick %%s\" \"%s\"\n\n"
 		"NAME %s\n"		"CMD send %%s\n\n"
 		"NAME %s\n"		"CMD query %%s\n\n",
 				_("Op"),
@@ -996,7 +1012,7 @@ child_handler (gpointer userdata)
 #endif
 
 void
-xchat_exec (char *cmd)
+xchat_exec (const char *cmd)
 {
 #ifdef WIN32
 	util_exec (cmd);
@@ -1009,6 +1025,20 @@ xchat_exec (char *cmd)
 #endif
 }
 
+void
+xchat_execv (char * const argv[])
+{
+#ifdef WIN32
+	util_execv (argv);
+#else
+	int pid = util_execv (argv);
+	if (pid != -1)
+	/* zombie avoiding system. Don't ask! it has to be like this to work
+      with zvt (which overrides the default handler) */
+		fe_timeout_add (1000, child_handler, GINT_TO_POINTER (pid));
+#endif
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1023,6 +1053,10 @@ main (int argc, char *argv[])
 	ret = fe_args (argc, argv);
 	if (ret != -1)
 		return ret;
+	
+#ifdef USE_DBUS
+	xchat_remote ();
+#endif
 
 	load_config ();
 
