@@ -81,9 +81,6 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #define USERNAMELEN 10
 #define HIDDEN_CHAR	8			/* invisible character for xtext */
 
-#define safe_strcpy(dest,src,len)	{strncpy((dest),(src),(len)); \
-												(dest)[len-1] = 0;}
-
 #if defined(ENABLE_NLS) && !defined(_)
 #  include <libintl.h>
 #  define _(x) gettext(x)
@@ -229,10 +226,23 @@ struct xchatprefs
 	unsigned int slist_skip;
 	unsigned int slist_select;
 	unsigned int filterbeep;
-	unsigned int beepmsg;
-	unsigned int beepchans;
-	unsigned int beephilight;
-	unsigned int flash_hilight;
+
+	unsigned int input_balloon_chans;
+	unsigned int input_balloon_hilight;
+	unsigned int input_balloon_priv;
+
+	unsigned int input_beep_chans;
+	unsigned int input_beep_hilight;
+	unsigned int input_beep_priv;
+
+	unsigned int input_flash_chans;
+	unsigned int input_flash_hilight;
+	unsigned int input_flash_priv;
+
+	unsigned int input_tray_chans;
+	unsigned int input_tray_hilight;
+	unsigned int input_tray_priv;
+
 	unsigned int truncchans;
 	unsigned int privmsgtab;
 	unsigned int irc_join_delay;
@@ -261,6 +271,10 @@ struct xchatprefs
 	unsigned int auto_indent;
 	unsigned int wordwrap;
 	unsigned int gui_input_spell;
+	unsigned int gui_tray;
+	unsigned int _gui_tray_blink;
+	unsigned int gui_tray_flags;
+	unsigned int gui_ulist_left;
 	unsigned int throttle;
 	unsigned int topicbar;
 	unsigned int hideuserlist;
@@ -341,8 +355,10 @@ typedef struct session
 	/* these are in the bottom-right menu */
 	unsigned int hide_join_part:1;	/* hide join & part messages? */
 	unsigned int beep:1;				/* beep enabled? */
+	unsigned int tray:1;				/* tray flash for this chan? */
 	unsigned int color_paste:1;
 	int done_away_check:1;	/* done checking for away status changes */
+	unsigned int lastlog_regexp:1;	/* this is a lastlog and using regexp */
 } session;
 
 struct msproxy_state_t
@@ -387,7 +403,7 @@ typedef struct server
 	void (*p_action)(struct server *, char *channel, char *act);
 	void (*p_notice)(struct server *, char *channel, char *text);
 	void (*p_topic)(struct server *, char *channel, char *topic);
-	void (*p_list_channels)(struct server *, char *arg);
+	void (*p_list_channels)(struct server *, char *arg, int min_users);
 	void (*p_change_nick)(struct server *, char *new_nick);
 	void (*p_names)(struct server *, char *channel);
 	void (*p_ping)(struct server *, char *to, char *timestring);
@@ -407,6 +423,8 @@ typedef struct server
 #ifdef USE_OPENSSL
 	SSL *ssl;
 	int ssl_do_connect_tag;
+#else
+	void *ssl;
 #endif
 	int childread;
 	int childwrite;
@@ -437,6 +455,7 @@ typedef struct server
 	time_t next_send;						/* cptr->since in ircu */
 	time_t prev_now;					/* previous now-time */
 	int sendq_len;						/* queue size */
+	int lag;								/* milliseconds */
 
 	struct session *front_session;	/* front-most window/tab */
 	struct session *server_session;	/* server window/tab */
@@ -476,6 +495,7 @@ typedef struct server
 	unsigned int have_idmsg:1;	/* freenode's IDENTIFY-MSG */
 	unsigned int have_except:1;	/* ban exemptions +e */
 	unsigned int using_cp1255:1;	/* encoding is CP1255/WINDOWS-1255? */
+	unsigned int using_irc:1;		/* encoding is "IRC" (CP1252/UTF-8 hybrid)? */
 	int use_who:1;				/* whether to use WHO command to get dcc_ip */
 #ifdef USE_OPENSSL
 	int use_ssl:1;					  /* is server SSL capable? */
@@ -515,4 +535,12 @@ struct popup
 /* CL: get a random int in the range [0..n-1]. DON'T use rand() % n, it gives terrible results. */
 #define RAND_INT(n) ((int)(rand() / (RAND_MAX + 1.0) * (n)))
 
+#if defined(WIN32) && GLIB_CHECK_VERSION(2,4,0)
+#define xchat_filename_from_utf8 g_locale_from_utf8
+#define xchat_filename_to_utf8 g_locale_to_utf8
+#else
+#define xchat_filename_from_utf8 g_filename_from_utf8
+#define xchat_filename_to_utf8 g_filename_to_utf8
+#endif
+
 #endif
