@@ -40,15 +40,18 @@
 #include "../common/text.h"
 #include "../common/cfgfiles.h"
 #include "../common/xchatc.h"
+#include "../common/plugin.h"
 #include "gtkutil.h"
 #include "maingui.h"
 #include "pixmaps.h"
+#include "joind.h"
 #include "xtext.h"
 #include "palette.h"
 #include "menu.h"
 #include "notifygui.h"
 #include "textgui.h"
 #include "fkeys.h"
+#include "plugin-tray.h"
 #include "urlgrab.h"
 
 #ifdef USE_XLIB
@@ -116,16 +119,21 @@ static char *arg_cfgdir = NULL;
 static gint arg_show_autoload = 0;
 static gint arg_show_config = 0;
 static gint arg_show_version = 0;
+static gint arg_minimize = 0;
 
 static const GOptionEntry gopt_entries[] = 
 {
- {"no-auto",	'a', 0, G_OPTION_ARG_NONE,		&arg_dont_autoconnect, N_("Don't auto connect to servers"), NULL},
- {"cfgdir",		'd', 0, G_OPTION_ARG_STRING,	&arg_cfgdir, N_("Use a different config directory"), "PATH"},
- {"no-plugins",'n', 0, G_OPTION_ARG_NONE,		&arg_skip_plugins, N_("Don't auto load any plugins"), NULL},
- {"plugindir",	'p', 0, G_OPTION_ARG_NONE,		&arg_show_autoload, N_("Show plugin auto-load directory"), NULL},
- {"configdir",	'u', 0, G_OPTION_ARG_NONE,		&arg_show_config, N_("Show user config directory"), NULL},
- {"url",			 0,  0, G_OPTION_ARG_STRING,	&arg_url, N_("Open an irc://server:port/channel URL"), "URL"},
- {"version",	'v', 0, G_OPTION_ARG_NONE,		&arg_show_version, N_("Show version information"), NULL},
+ {"no-auto",	'a', 0, G_OPTION_ARG_NONE,	&arg_dont_autoconnect, N_("Don't auto connect to servers"), NULL},
+ {"cfgdir",	'd', 0, G_OPTION_ARG_STRING,	&arg_cfgdir, N_("Use a different config directory"), "PATH"},
+ {"no-plugins",	'n', 0, G_OPTION_ARG_NONE,	&arg_skip_plugins, N_("Don't auto load any plugins"), NULL},
+ {"plugindir",	'p', 0, G_OPTION_ARG_NONE,	&arg_show_autoload, N_("Show plugin auto-load directory"), NULL},
+ {"configdir",	'u', 0, G_OPTION_ARG_NONE,	&arg_show_config, N_("Show user config directory"), NULL},
+ {"url",	 0,  0, G_OPTION_ARG_STRING,	&arg_url, N_("Open an irc://server:port/channel URL"), "URL"},
+#ifndef WIN32
+ {"existing",	'e', 0, G_OPTION_ARG_NONE,	&arg_existing, N_("Open URL in an existing XChat"), NULL},
+#endif
+ {"minimize",	 0,  0, G_OPTION_ARG_INT,	&arg_minimize, N_("Begin minimized. Level 0=Normal 1=Iconified 2=Tray"), N_("level")},
+ {"version",	'v', 0, G_OPTION_ARG_NONE,	&arg_show_version, N_("Show version information"), NULL},
  {NULL}
 };
 
@@ -143,7 +151,7 @@ fe_args (int argc, char *argv[])
 
 	context = g_option_context_new (NULL);
 	g_option_context_add_main_entries (context, gopt_entries, GETTEXT_PACKAGE);
-	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+	g_option_context_add_group (context, gtk_get_option_group (FALSE));
 	g_option_context_parse (context, &argc, &argv, &error);
 
 	if (error)
@@ -337,6 +345,23 @@ log_handler (const gchar   *log_domain,
 
 #endif
 
+/* install tray stuff */
+
+static int
+fe_idle (gpointer data)
+{
+	session *sess = sess_list->data;
+
+	plugin_add (sess, NULL, NULL, tray_plugin_init, tray_plugin_deinit, NULL, FALSE);
+
+	if (arg_minimize == 1)
+		gtk_window_iconify (GTK_WINDOW (sess->gui->window));
+	else if (arg_minimize == 2)
+		tray_toggle_visibility ();
+
+	return 0;
+}
+
 void
 fe_new_window (session *sess, int focus)
 {
@@ -360,6 +385,9 @@ fe_new_window (session *sess, int focus)
 	g_log_set_handler ("Gdk", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
 	g_log_set_handler ("Gtk", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
 #endif
+
+	if (!sess_list->next)
+		g_idle_add (fe_idle, NULL);
 }
 
 void
@@ -453,9 +481,10 @@ void
 fe_set_hilight (struct session *sess)
 {
 	if (sess->gui->is_tab)
-		fe_set_tab_color (sess, 3, TRUE);	/* blue, with taskbar flash */
-	else
-		fe_set_tab_color (sess, -1, TRUE);	/* taskbar flash only */
+		fe_set_tab_color (sess, 3);	/* set tab to blue */
+
+	if (prefs.input_flash_hilight)
+		fe_flash_window (sess); /* taskbar flash */
 }
 
 static void
@@ -574,9 +603,9 @@ fe_print_text (struct session *sess, char *text)
 	{
 		sess->new_data = TRUE;
 		if (sess->msg_said)
-			fe_set_tab_color (sess, 2, FALSE);
+			fe_set_tab_color (sess, 2);
 		else
-			fe_set_tab_color (sess, 1, FALSE);
+			fe_set_tab_color (sess, 1);
 	}
 }
 
@@ -586,17 +615,42 @@ fe_beep (void)
 	gdk_beep ();
 }
 
-void
-fe_lastlog (session *sess, session *lastlog_sess, char *sstr)
+#ifndef WIN32
+static int
+lastlog_regex_cmp (char *a, regex_t *reg)
 {
+	return !regexec (reg, a, 1, NULL, REG_NOTBOL);
+}
+#endif
+
+void
+fe_lastlog (session *sess, session *lastlog_sess, char *sstr, gboolean regexp)
+{
+#ifndef WIN32
+	regex_t reg;
+#endif
+
 	if (gtk_xtext_is_empty (sess->res->buffer))
 	{
 		PrintText (lastlog_sess, _("Search buffer is empty.\n"));
+		return;
 	}
-	else
+
+	if (!regexp)
 	{
-		gtk_xtext_lastlog (lastlog_sess->res->buffer, sess->res->buffer, sstr);
+		gtk_xtext_lastlog (lastlog_sess->res->buffer, sess->res->buffer,
+								 (void *) nocasestrstr, sstr);
+		return;
 	}
+
+#ifndef WIN32
+	if (regcomp (&reg, sstr, REG_ICASE | REG_EXTENDED | REG_NOSUB) == 0)
+	{
+		gtk_xtext_lastlog (lastlog_sess->res->buffer, sess->res->buffer,
+								 (void *) lastlog_regex_cmp, &reg);
+		regfree (&reg);
+	}
+#endif
 }
 
 void
@@ -716,9 +770,9 @@ fe_ctrl_gui (session *sess, int action, int arg)
 	case 2:
 		mg_bring_tofront_sess (sess); break;
 	case 3:
-		fe_set_tab_color (sess, -1, TRUE); break; /* flash */
+		fe_flash_window (sess); break;
 	case 4:
-		fe_set_tab_color (sess, arg, FALSE); break;
+		fe_set_tab_color (sess, arg); break;
 	case 5:
 		gtk_window_iconify (GTK_WINDOW (sess->gui->window)); break;
 	case 6:
@@ -726,6 +780,9 @@ fe_ctrl_gui (session *sess, int action, int arg)
 		break;
 	case 7:
 		mg_detach (sess, arg);	/* arg: 0=toggle 1=detach 2=attach */
+		break;
+	case 8:
+		setup_apply_real (TRUE, TRUE);
 	}
 }
 
@@ -843,34 +900,64 @@ fe_set_inputbox_contents (session *sess, char *text)
 	}
 }
 
+#ifndef WIN32
+
+static gboolean
+try_browser (const char *browser, const char *arg, const char *url)
+{
+	const char *argv[4];
+	char *path;
+
+	path = g_find_program_in_path (browser);
+	if (!path)
+		return 0;
+
+	argv[0] = path;
+	argv[1] = url;
+	argv[2] = NULL;
+	if (arg)
+	{
+		argv[1] = arg;
+		argv[2] = url;
+		argv[3] = NULL;
+	}
+	xchat_execv (argv);
+	g_free (path);
+	return 1;
+}
+
+#endif
+
 static void
 fe_open_url_locale (const char *url)
 {
 #ifdef WIN32
 	ShellExecute (0, "open", url, NULL, NULL, SW_SHOWNORMAL);
 #else
-	char tbuf[512], *moz;
-
-	/* gnome 2.4+ has this */
-	moz = g_find_program_in_path ("gnome-open");
-	if (moz)
-	{
-		snprintf (tbuf, sizeof (tbuf), "%s %s", moz, url);
-		g_free (moz);
-		xchat_exec (tbuf);
+	/* universal desktop URL opener (from xdg-utils). Supports gnome,kde,xfce4. */
+	if (try_browser ("xdg-open", NULL, url))
 		return;
+
+	/* try to detect GNOME */
+	if (g_getenv ("GNOME_DESKTOP_SESSION_ID"))
+	{
+		if (try_browser ("gnome-open", NULL, url)) /* Gnome 2.4+ has this */
+			return;
 	}
 
-	moz = g_find_program_in_path ("firefox");
-	if (moz)
+	/* try to detect KDE */
+	if (g_getenv ("KDE_FULL_SESSION"))
 	{
-		snprintf (tbuf, sizeof (tbuf), "%s %s", moz, url);
-		g_free (moz);
-	} else
-	{
-		snprintf (tbuf, sizeof (tbuf), "mozilla -remote 'openURL(%s)'", url);
+		if (try_browser ("kfmclient", "exec", url))
+			return;
 	}
-	xchat_exec (tbuf);
+
+	/* everything failed, what now? just try firefox */
+	if (try_browser ("firefox", NULL, url))
+		return;
+
+	/* fresh out of ideas... */
+	try_browser ("mozilla", NULL, url);
 #endif
 }
 
@@ -926,7 +1013,7 @@ fe_server_event (server *serv, int type, int arg)
 				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_JOIN], 1);
 				/* if number of auto-join channels is zero, open joind */
 				if (arg == 0)
-					joind (0, serv);
+					joind_open (serv);
 				break;
 
 			case FE_SE_DISCONNECT:
@@ -935,9 +1022,20 @@ fe_server_event (server *serv, int type, int arg)
 				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_DISCONNECT], 0);
 				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_JOIN], 0);
 				/* close the join-dialog, if one exists */
-				joind (1, serv);
+				joind_close (serv);
 			}
 		}
 		list = list->next;
 	}
 }
+
+void
+fe_get_file (const char *title, char *initial,
+				 void (*callback) (void *userdata, char *file), void *userdata,
+				 int flags)
+				
+{
+	/* OK: Call callback once per file, then once more with file=NULL. */
+	/* CANCEL: Call callback once with file=NULL. */
+	gtkutil_file_req (title, callback, userdata, initial, flags | FRF_FILTERISINITIAL);
+}
