@@ -20,23 +20,17 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include "fe-gtk.h"
 #include "../common/xchat.h"
-#include "gtkutil.h"
+#include "../common/fe.h"
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk-pixbuf/gdk-pixdata.h>
+#include <gtk/gtkstock.h>
 
-
-#include "../pixmaps/xchat.h"
-#include "../pixmaps/book.h"
-
-#include "../pixmaps/red.h"
-#include "../pixmaps/purple.h"
-#include "../pixmaps/op.h"
-#include "../pixmaps/hop.h"
-#include "../pixmaps/voice.h"
+#include "../pixmaps/inline_pngs.h"
 
 GdkPixbuf *pix_xchat;
 GdkPixbuf *pix_book;
@@ -47,6 +41,15 @@ GdkPixbuf *pix_op;
 GdkPixbuf *pix_hop;
 GdkPixbuf *pix_voice;
 
+GdkPixbuf *pix_tray_msg;
+GdkPixbuf *pix_tray_hilight;
+GdkPixbuf *pix_tray_file;
+
+GdkPixbuf *pix_channel;
+GdkPixbuf *pix_dialog;
+GdkPixbuf *pix_server;
+GdkPixbuf *pix_util;
+
 
 static GdkPixmap *
 pixmap_load_from_file_real (char *file)
@@ -76,36 +79,45 @@ pixmap_load_from_file (char *filename)
 	if (pix == NULL)
 	{
 		strcpy (buf, "Cannot open:\n\n");
-		strcpy (buf + 14, filename);
-		gtkutil_simpledialog (buf);
+		strncpy (buf + 14, filename, sizeof (buf) - 14);
+		buf[sizeof (buf) - 1] = 0;
+		fe_message (buf, FE_MSG_ERROR);
 	}
 
 	return pix;
 }
 
+#define LOADPIX(vv,pp,ff) \
+	vv = gdk_pixbuf_new_from_file (XCHATSHAREDIR"/"ff, 0); \
+	if (!vv) \
+		vv = gdk_pixbuf_new_from_inline (-1, pp, FALSE, 0);
+
+#define LOADPIX_DISKONLY(vv,ff) \
+	vv = gdk_pixbuf_new_from_file (XCHATSHAREDIR"/"ff, 0);
+
+#define EXT ".png"
+
 void
 pixmaps_init (void)
 {
 	pix_book = gdk_pixbuf_new_from_inline (-1, bookpng, FALSE, 0);
 	pix_xchat = gdk_pixbuf_new_from_inline (-1, xchatpng, FALSE, 0);
 
-	pix_purple = gdk_pixbuf_new_from_file (XCHATSHAREDIR"/purple.png", 0);
-	if (!pix_purple)
-		pix_purple = gdk_pixbuf_new_from_inline (-1, purplepng, FALSE, 0);
+	/* userlist icons, with inlined defaults */
+	LOADPIX (pix_hop, hoppng, "hop"EXT);
+	LOADPIX (pix_purple, purplepng, "purple"EXT);
+	LOADPIX (pix_red, redpng, "red"EXT);
+	LOADPIX (pix_op, oppng, "op"EXT);
+	LOADPIX (pix_voice, voicepng, "voice"EXT);
 
-	pix_red = gdk_pixbuf_new_from_file (XCHATSHAREDIR"/red.png", 0);
-	if (!pix_red)
-		pix_red = gdk_pixbuf_new_from_inline (-1, redpng, FALSE, 0);
+	/* tray icons, with inlined defaults */
+	LOADPIX (pix_tray_msg, traymsgpng, "message"EXT);
+	LOADPIX (pix_tray_hilight, trayhilightpng, "highlight"EXT);
+	LOADPIX (pix_tray_file, trayfilepng, "fileoffer"EXT);
 
-	pix_op = gdk_pixbuf_new_from_file (XCHATSHAREDIR"/op.png", 0);
-	if (!pix_op)
-		pix_op = gdk_pixbuf_new_from_inline (-1, oppng, FALSE, 0);
-
-	pix_hop = gdk_pixbuf_new_from_file (XCHATSHAREDIR"/hop.png", 0);
-	if (!pix_hop)
-		pix_hop = gdk_pixbuf_new_from_inline (-1, hoppng, FALSE, 0);
-
-	pix_voice = gdk_pixbuf_new_from_file (XCHATSHAREDIR"/voice.png", 0);
-	if (!pix_voice)
-		pix_voice = gdk_pixbuf_new_from_inline (-1, voicepng, FALSE, 0);
+	/* treeview icons, no defaults, load from disk only */
+	LOADPIX_DISKONLY (pix_channel,	"channel"EXT);
+	LOADPIX_DISKONLY (pix_dialog,		"dialog"EXT);
+	LOADPIX_DISKONLY (pix_server,		"server"EXT);
+	LOADPIX_DISKONLY (pix_util,		"util"EXT);
 }
