@@ -72,6 +72,7 @@ enum
 	ST_END,
 	ST_TOGGLE,
 	ST_TOGGLR,
+	ST_3OGGLE,
 	ST_ENTRY,
 	ST_EFONT,
 	ST_EFILE,
@@ -81,7 +82,8 @@ enum
 	ST_NUMBER,
 	ST_HSCALE,
 	ST_HEADER,
-	ST_LABEL
+	ST_LABEL,
+	ST_ALERTHEAD
 };
 
 typedef struct
@@ -275,6 +277,48 @@ static const setting filexfer_settings[] =
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
+static const int balloonlist[3] =
+{
+	P_OFFINTNL(input_balloon_chans), P_OFFINTNL(input_balloon_priv), P_OFFINTNL(input_balloon_hilight)
+};
+
+static const int trayblinklist[3] =
+{
+	P_OFFINTNL(input_tray_chans), P_OFFINTNL(input_tray_priv), P_OFFINTNL(input_tray_hilight)
+};
+
+static const int taskbarlist[3] =
+{
+	P_OFFINTNL(input_flash_chans), P_OFFINTNL(input_flash_priv), P_OFFINTNL(input_flash_hilight)
+};
+
+static const int beeplist[3] =
+{
+	P_OFFINTNL(input_beep_chans), P_OFFINTNL(input_beep_priv), P_OFFINTNL(input_beep_hilight)
+};
+
+static const setting alert_settings[] =
+{
+	{ST_HEADER,	N_("Alerts"),0,0,0},
+
+	{ST_ALERTHEAD},
+	{ST_3OGGLE, N_("Show tray balloons on:"), 0, 0, (void *)balloonlist, 0},
+	{ST_3OGGLE, N_("Blink tray icon on:"), 0, 0, (void *)trayblinklist, 0},
+	{ST_3OGGLE, N_("Blink task bar on:"), 0, 0, (void *)taskbarlist, 0},
+	{ST_3OGGLE, N_("Make a beep sound on:"), 0, 0, (void *)beeplist, 0},
+
+	{ST_TOGGLE,	N_("Enable system tray icon"), P_OFFINTNL(gui_tray), 0, 0, 0},
+
+	{ST_HEADER,	N_("Highlighted Messages"),0,0,0},
+	{ST_LABEL,	N_("Highlighted messages are ones where your nickname is mentioned, but also:"), 0, 0, 0, 1},
+
+	{ST_ENTRY,	N_("Extra words to highlight:"), P_OFFSETNL(irc_extra_hilight), 0, 0, sizeof prefs.irc_extra_hilight},
+	{ST_ENTRY,	N_("Nick names not to highlight:"), P_OFFSETNL(irc_no_hilight), 0, 0, sizeof prefs.irc_no_hilight},
+	{ST_ENTRY,	N_("Nick names to always highlight:"), P_OFFSETNL(irc_nick_hilight), 0, 0, sizeof prefs.irc_nick_hilight},
+	{ST_LABEL,	N_("Separate multiple words with commas.")},
+	{ST_END, 0, 0, 0, 0, 0}
+};
+
 static const setting general_settings[] =
 {
 	{ST_HEADER,	N_("Default Messages"),0,0,0},
@@ -287,24 +331,6 @@ static const setting general_settings[] =
 					N_("Announce your away messages to all channels"), 0, 0},
 	{ST_TOGGLE,	N_("Show away once"), P_OFFINTNL(show_away_once), N_("Show identical away messages only once"), 0, 0},
 	{ST_TOGGLE,	N_("Automatically unmark away"), P_OFFINTNL(auto_unmark_away), N_("Unmark yourself as away before sending messages"), 0, 0},
-#if 0
-#ifndef WIN32
-	{ST_LABEL,	N_("(Can be a text file relative to ~/.xchat2/).")},
-#else
-	{ST_LABEL,	N_("(Can be a text file relative to config dir).")},
-#endif
-#endif
-
-	{ST_HEADER,	N_("Alerts"),0,0,0},
-#if defined(WIN32) || defined(USE_XLIB)
-	{ST_TOGGLE,	N_("Flash taskbar on highlighted messages"), P_OFFINTNL(flash_hilight), 0, 0, 0},
-#endif
-	{ST_TOGGLE,	N_("Beep on highlighted messages"), P_OFFINTNL(beephilight), 0, 0, 0},
-	{ST_TOGGLE,	N_("Beep on private messages"), P_OFFINTNL(beepmsg), 0, 0, 0},
-	{ST_TOGGLE,	N_("Beep on channel messages"), P_OFFINTNL(beepchans), 0, 0, 0},
-	{ST_ENTRY,	N_("Extra words to highlight on:"), P_OFFSETNL(irc_extra_hilight), 0, 0, sizeof prefs.irc_extra_hilight},
-	{ST_ENTRY,	N_("Nicks not to highlight on:"), P_OFFSETNL(irc_no_hilight), 0, 0, sizeof prefs.irc_no_hilight},
-	{ST_LABEL,	N_("Separate multiple words with commas.")},
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
@@ -387,11 +413,79 @@ static const setting network_settings[] =
 
 #define setup_get_str(pr,set) (((char *)pr)+set->offset)
 #define setup_get_int(pr,set) *(((int *)pr)+set->offset)
+#define setup_get_int3(pr,off) *(((int *)pr)+off) 
 
 #define setup_set_int(pr,set,num) *((int *)pr+set->offset)=num
 #define setup_set_str(pr,set,str) strcpy(((char *)pr)+set->offset,str)
 
 
+static void
+setup_3oggle_cb (GtkToggleButton *but, unsigned int *setting)
+{
+	*setting = but->active;
+}
+
+static void
+setup_headlabel (GtkWidget *tab, int row, int col, char *text)
+{
+	GtkWidget *label;
+	char buf[128];
+	char *sp;
+
+	snprintf (buf, sizeof (buf), "<b><span size=\"smaller\">%s</span></b>", text);
+	sp = strchr (buf + 17, ' ');
+	if (sp)
+		*sp = '\n';
+
+	label = gtk_label_new (NULL);
+	gtk_label_set_markup (GTK_LABEL (label), buf);
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_table_attach (GTK_TABLE (tab), label, col, col + 1, row, row + 1, 0, 0, 4, 0);
+}
+
+static void
+setup_create_alert_header (GtkWidget *tab, int row, const setting *set)
+{
+	setup_headlabel (tab, row, 3, _("Channel Message"));
+	setup_headlabel (tab, row, 4, _("Private Message"));
+	setup_headlabel (tab, row, 5, _("Highlighted Message"));
+}
+
+/* makes 3 toggles side-by-side */
+
+static void
+setup_create_3oggle (GtkWidget *tab, int row, const setting *set)
+{
+	GtkWidget *label, *wid;
+	int *offsets = (int *)set->list;
+
+	label = gtk_label_new (_(set->label));
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_table_attach (GTK_TABLE (tab), label, 2, 3, row, row + 1,
+							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, LABEL_INDENT, 0);
+
+	wid = gtk_check_button_new ();
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid),
+											setup_get_int3 (&setup_prefs, offsets[0]));
+	g_signal_connect (G_OBJECT (wid), "toggled",
+							G_CALLBACK (setup_3oggle_cb), ((int *)&setup_prefs) + offsets[0]);
+	gtk_table_attach (GTK_TABLE (tab), wid, 3, 4, row, row + 1, 0, 0, 0, 0);
+
+	wid = gtk_check_button_new ();
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid),
+											setup_get_int3 (&setup_prefs, offsets[1]));
+	g_signal_connect (G_OBJECT (wid), "toggled",
+							G_CALLBACK (setup_3oggle_cb), ((int *)&setup_prefs) + offsets[1]);
+	gtk_table_attach (GTK_TABLE (tab), wid, 4, 5, row, row + 1, 0, 0, 0, 0);
+
+	wid = gtk_check_button_new ();
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid),
+											setup_get_int3 (&setup_prefs, offsets[2]));
+	g_signal_connect (G_OBJECT (wid), "toggled",
+							G_CALLBACK (setup_3oggle_cb), ((int *)&setup_prefs) + offsets[2]);
+	gtk_table_attach (GTK_TABLE (tab), wid, 5, 6, row, row + 1, 0, 0, 0, 0);
+}
+
 static void
 setup_toggle_cb (GtkToggleButton *but, const setting *set)
 {
@@ -776,14 +870,40 @@ setup_browsefont_cb (GtkWidget *button, GtkWidget *entry)
 static void
 setup_entry_cb (GtkEntry *entry, setting *set)
 {
-	setup_set_str (&setup_prefs, set, entry->text);
+	int size;
+	int pos;
+	int len = strlen (entry->text);
+	unsigned char *p = entry->text;
+
+	/* need to truncate? */
+	if (len >= set->extra)
+	{
+		len = pos = 0;
+		while (1)
+		{
+			size = g_utf8_skip [*p];
+			len += size;
+			p += size;
+			/* truncate to "set->extra" BYTES */
+			if (len >= set->extra)
+			{
+				gtk_editable_delete_text (GTK_EDITABLE (entry), pos, -1);
+				break;
+			}
+			pos++;
+		}
+	}
+	else
+	{
+		setup_set_str (&setup_prefs, set, entry->text);
+	}
 }
 
 static void
 setup_create_label (GtkWidget *table, int row, const setting *set)
 {
 	gtk_table_attach (GTK_TABLE (table), setup_create_italic_label (_(set->label)),
-							3, 5, row, row + 1, GTK_FILL,
+							set->extra ? 1 : 3, 5, row, row + 1, GTK_FILL,
 							GTK_SHRINK | GTK_FILL, 0, 0);
 }
 
@@ -924,6 +1044,9 @@ setup_create_page (const setting *set)
 			wid = setup_create_toggleL (tab, row, &set[i]);
 			do_disable = set[i].extra;
 			break;
+		case ST_3OGGLE:
+			setup_create_3oggle (tab, row, &set[i]);
+			break;
 		case ST_MENU:
 			setup_create_menu (tab, row, &set[i]);
 			break;
@@ -941,6 +1064,8 @@ setup_create_page (const setting *set)
 		case ST_LABEL:
 			setup_create_label (tab, row, &set[i]);
 			break;
+		case ST_ALERTHEAD:
+			setup_create_alert_header (tab, row, &set[i]);
 		}
 
 		/* will this toggle disable the "next" widget? */
@@ -971,7 +1096,7 @@ setup_create_page (const setting *set)
 
 	if (set == logging_settings)
 	{
-		GtkWidget *but = gtk_button_new_with_label ("Open Data Folder");
+		GtkWidget *but = gtk_button_new_with_label (_("Open Data Folder"));
 		gtk_box_pack_start (GTK_BOX (left), but, 0, 0, 0);
 		g_signal_connect (G_OBJECT (but), "clicked",
 								G_CALLBACK (open_data_cb), 0);
@@ -1517,6 +1642,7 @@ static const char *const cata[] =
 		N_("Colors"),
 		NULL,
 	N_("Chatting"),
+		N_("Alerts"),
 		N_("General"),
 		N_("Logging"),
 		N_("Sound"),
@@ -1541,12 +1667,12 @@ setup_create_pages (GtkWidget *box)
 	setup_add_page (cata[3], book, setup_create_page (userlist_settings));
 	setup_add_page (cata[4], book, setup_create_page (tabs_settings));
 	setup_add_page (cata[5], book, setup_create_color_page ());
-	setup_add_page (cata[8], book, setup_create_page (general_settings));
-	setup_add_page (cata[9], book, setup_create_page (logging_settings));
-	setup_add_page (cata[10], book, setup_create_sound_page ());
-	/*setup_add_page (cata[11], book, setup_create_page (advanced_settings));*/
-	setup_add_page (cata[13], book, setup_create_page (network_settings));
-	setup_add_page (cata[14], book, setup_create_page (filexfer_settings));
+	setup_add_page (cata[8], book, setup_create_page (alert_settings));
+	setup_add_page (cata[9], book, setup_create_page (general_settings));
+	setup_add_page (cata[10], book, setup_create_page (logging_settings));
+	setup_add_page (cata[11], book, setup_create_sound_page ());
+	setup_add_page (cata[14], book, setup_create_page (network_settings));
+	setup_add_page (cata[15], book, setup_create_page (filexfer_settings));
 
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (book), FALSE);
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (book), FALSE);
@@ -1722,44 +1848,12 @@ unslash (char *dir)
 	}
 }
 
-static void
-setup_apply (struct xchatprefs *pr)
+void
+setup_apply_real (int new_pix, int do_ulist)
 {
 	GSList *list;
-	int done_main = FALSE;
 	session *sess;
-	int new_pix = FALSE;
-	int noapply = FALSE;
-	int do_ulist = FALSE;
-
-	if (strcmp (pr->background, prefs.background) != 0)
-		new_pix = TRUE;
-
-#define DIFF(a) (pr->a != prefs.a)
-
-	if (DIFF (paned_userlist))
-		noapply = TRUE;
-	if (DIFF (lagometer))
-		noapply = TRUE;
-	if (DIFF (throttlemeter))
-		noapply = TRUE;
-	if (DIFF (showhostname_in_userlist))
-		noapply = TRUE;
-	if (DIFF (tab_small))
-		noapply = TRUE;
-	if (DIFF (tab_sort))
-		noapply = TRUE;
-	if (DIFF (use_server_tab))
-		noapply = TRUE;
-	if (DIFF (style_namelistgad))
-		noapply = TRUE;
-	if (DIFF (truncchans))
-		noapply = TRUE;
-
-	if (color_change || (DIFF (away_size_max)) || (DIFF (away_track)))
-		do_ulist = TRUE;
-
-	memcpy (&prefs, pr, sizeof (prefs));
+	int done_main = FALSE;
 
 	/* remove trailing slashes */
 	unslash (prefs.dccdir);
@@ -1806,6 +1900,53 @@ setup_apply (struct xchatprefs *pr)
 	}
 
 	mg_apply_setup ();
+	tray_apply_setup ();
+}
+
+static void
+setup_apply (struct xchatprefs *pr)
+{
+	int new_pix = FALSE;
+	int noapply = FALSE;
+	int do_ulist = FALSE;
+
+	if (strcmp (pr->background, prefs.background) != 0)
+		new_pix = TRUE;
+
+#define DIFF(a) (pr->a != prefs.a)
+
+	if (DIFF (paned_userlist))
+		noapply = TRUE;
+	if (DIFF (lagometer))
+		noapply = TRUE;
+	if (DIFF (throttlemeter))
+		noapply = TRUE;
+	if (DIFF (showhostname_in_userlist))
+		noapply = TRUE;
+	if (DIFF (tab_small))
+		noapply = TRUE;
+	if (DIFF (tab_sort))
+		noapply = TRUE;
+	if (DIFF (use_server_tab))
+		noapply = TRUE;
+	if (DIFF (style_namelistgad))
+		noapply = TRUE;
+	if (DIFF (truncchans))
+		noapply = TRUE;
+
+	if (color_change || (DIFF (away_size_max)) || (DIFF (away_track)))
+		do_ulist = TRUE;
+
+	if (pr->tabs_position < 2 && pr->tabs_position != 4 &&
+		 pr->tab_layout == 2 && pr->tabs_position != prefs.tabs_position)
+		fe_message (_("You cannot place the tree on the top or bottom!\n"
+						"Please change to the <b>Tabs</b> layout in the <b>View</b>"
+						" menu first."),
+						FE_MSG_WARN | FE_MSG_MARKUP);
+
+	memcpy (&prefs, pr, sizeof (prefs));
+
+	setup_apply_real (new_pix, do_ulist);
 
 	if (noapply)
 		fe_message (_("Some settings were changed that require a"
