@@ -584,7 +584,7 @@ fe_set_hilight (struct session *sess)
 {
 }
 void
-fe_set_tab_color (struct session *sess, int col, int flash)
+fe_set_tab_color (struct session *sess, int col)
 {
 }
 void
@@ -759,7 +759,7 @@ fe_userlist_hide (session * sess)
 {
 }
 void
-fe_lastlog (session * sess, session * lastlog_sess, char *sstr)
+fe_lastlog (session * sess, session * lastlog_sess, char *sstr, gboolean regexp)
 {
 }
 void
@@ -775,6 +775,10 @@ fe_set_away (server *serv)
 {
 }
 void
+fe_set_color_paste (session *sess, int status)
+{
+}
+void
 fe_serverlist_open (session *sess)
 {
 }
@@ -827,8 +831,9 @@ void fe_open_url (const char *url)
 void fe_menu_del (menu_entry *me)
 {
 }
-void fe_menu_add (menu_entry *me)
+char *fe_menu_add (menu_entry *me)
 {
+	return NULL;
 }
 void fe_menu_update (menu_entry *me)
 {
@@ -840,3 +845,17 @@ void
 fe_server_event (server *serv, int type, int arg)
 {
 }
+void
+fe_flash_window (struct session *sess)
+{
+}
+void fe_get_file (const char *title, char *initial,
+				 void (*callback) (void *userdata, char *file), void *userdata,
+				 int flags)
+{
+}
+void fe_tray_set_flash (const char *filename1, const char *filename2, int timeout){}
+void fe_tray_set_file (const char *filename){}
+void fe_tray_set_icon (int icon){}
+void fe_tray_set_tooltip (const char *text){}
+void fe_tray_set_balloon (const char *title, const char *text){}
